from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.contrib import messages

from tsuru_autoscale.wizard import forms
from tsuru_autoscale.wizard import client
from tsuru_autoscale.datasource import client as dclient

import requests
import os
import urllib
import json


def tsuru_host():
    return os.environ.get("TSURU_HOST", "")


def app_info(name, token):
    url = "{}/apps/{}".format(tsuru_host(), name)
    headers = {"Authorization": token}
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return response.json()
    return None


def process_list(instance_name, token):
    token = urllib.unquote(token)
    token = "bearer {}".format(token)
    app = app_info(instance_name, token)
    process = set()

    for u in app.get('units', []):
        process.add(u['ProcessName'])

    p_list = []
    for u in list(process):
        p_list.append((u, u))

    return p_list


def get_or_create_tsuru_instance(instance_name, token):
    token = urllib.unquote(token)
    token = "bearer {}".format(token)
    url = "{}/services/instances/{}".format(tsuru_host(), instance_name)
    headers = {"Authorization": token}
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return

    app = app_info(instance_name, token)
    url = "{}/services/instances".format(tsuru_host(), instance_name)
    headers = {"Authorization": token}
    data = {"service_name": "autoscale", "name": instance_name, "owner": app["teamowner"]}
    response = requests.post(url, headers=headers, data=json.dumps(data))

    url = "{}/services/{}/instances/{}/{}".format(tsuru_host(), "autoscale", instance_name, instance_name)
    headers = {"Authorization": token}
    response = requests.put(url, headers=headers)


def new(request, instance=None):
    token = request.session.get('tsuru_token').split(" ")[-1]

    dlist = [(d["Name"], d["Name"]) for d in dclient.list(token).json()]

    scale_up_form = forms.ScaleForm(request.POST or None, prefix="scale_up", initial={"operator": ">"})
    scale_up_form.fields['metric'].choices = dlist

    scale_down_form = forms.ScaleForm(request.POST or None, prefix="scale_down", initial={"operator": "<"})
    scale_down_form.fields['metric'].choices = dlist

    config_form = forms.ConfigForm(request.POST or None)

    p_list = process_list(instance, token)
    config_form.fields['process'].choices = p_list

    if scale_up_form.is_valid() and scale_down_form.is_valid() and config_form.is_valid():
        get_or_create_tsuru_instance(instance, token)
        config_data = {
            "name": instance,
            "minUnits": config_form.cleaned_data["min"],
            "scaleUp": scale_up_form.cleaned_data,
            "scaleDown": scale_down_form.cleaned_data,
            "process": config_form.cleaned_data["process"],
        }
        client.new(config_data, token)
        messages.success(request, u"Auto scale saved.")
        url = "{}?TSURU_TOKEN={}".format(reverse("app-info", args=[instance]), urllib.quote(token))
        return redirect(url)

    token = urllib.quote(token)
    context = {
        "scale_up_form": scale_up_form,
        "scale_down_form": scale_down_form,
        "config_form": config_form,
        "token": token,
    }

    return render(request, "wizard/index.html", context)


def remove(request, instance):
    token = request.session.get('tsuru_token').split(" ")[-1]
    client.remove(instance, token)
    messages.success(request, u"Auto scale {} removed.".format(instance))
    url = "{}?TSURU_TOKEN={}".format(reverse("app-info", args=[instance]), urllib.quote(token))
    return redirect(url)



from django import forms


OPERATOR_CHOICES = (
    ('>', '>'),
    ('>=', '>='),
    ('<=', '<='),
    ('<', '<'),
    ('!=', '!='),
)


AGGREGATOR_CHOICES = (
    ('avg', 'average'),
    ('max', 'maximum'),
)


class ScaleForm(forms.Form):
    metric = forms.ChoiceField()
    aggregator = forms.ChoiceField(AGGREGATOR_CHOICES)
    operator = forms.ChoiceField(OPERATOR_CHOICES)
    value = forms.CharField()
    step = forms.CharField(label=u'Step (in units)')
    wait = forms.IntegerField(label=u'Wait time (in seconds)')


class ConfigForm(forms.Form):
    min = forms.IntegerField(label=u'Start with units')
    process = forms.ChoiceField()



import os
import json

import requests


def host():
    return os.environ.get("AUTOSCALE_HOST", "")


def new(data, token):
    url = "{}/wizard".format(host())
    headers = {"Authorization": token}
    response = requests.post(url, data=json.dumps(data), headers=headers)
    return response


def get(name, token):
    url = "{}/wizard/{}".format(host(), name)
    headers = {"Authorization": token}
    response = requests.get(url, headers=headers)
    return response


def remove(name, token):
    url = "{}/wizard/{}".format(host(), name)
    headers = {"Authorization": token}
    response = requests.delete(url, headers=headers)
    return response


def events(name, token):
    url = "{}/wizard/{}/events".format(host(), name)
    headers = {"Authorization": token}
    response = requests.get(url, headers=headers)
    return response



from django.test import TestCase
from django.core.urlresolvers import reverse

import mock


class IndexTestCase(TestCase):
    @mock.patch("tsuru_autoscale.instance.client.list")
    def test_index(self, list_mock):
        url = "{}?TSURU_TOKEN=bla".format(reverse("app-info", args=["app"]))
        response = self.client.get(url)
        self.assertTemplateUsed(response, "app/index.html")

    @mock.patch("tsuru_autoscale.instance.client.list")
    def test_index_instance_not_found(self, list_mock):
        response_mock = mock.Mock()
        response_mock.json.return_value = None
        list_mock.return_value = response_mock

        url = "{}?TSURU_TOKEN=bla".format(reverse("app-info", args=["app"]))
        response = self.client.get(url)

        self.assertTemplateUsed(response, "app/index.html")



from django.test import TestCase
from django.core.urlresolvers import reverse

from tsuru_autoscale.instance import client

import httpretty
import mock

import os


class ListTestCase(TestCase):
    @mock.patch("tsuru_autoscale.instance.client.list")
    def test_list(self, list_mock):
        url = "{}?TSURU_TOKEN=bla".format(reverse("instance-list"))
        response = self.client.get(url)

        self.assertTemplateUsed(response, "instance/list.html")
        self.assertIn('list', response.context)
        list_mock.assert_called_with("bla")


class GetTestCase(TestCase):
    @mock.patch("tsuru_autoscale.instance.client.alarms_by_instance")
    @mock.patch("tsuru_autoscale.instance.client.get")
    def test_get_without_alamrs(self, list_mock, alarms_by_instance_mock):
        json_mock = mock.Mock()
        json_mock.json.return_value = {"Name": "instance"}
        list_mock.return_value = json_mock

        aresponse = mock.Mock()
        aresponse.json.return_value = None
        alarms_by_instance_mock.return_value = aresponse

        url = "{}?TSURU_TOKEN=bla".format(reverse("instance-get", args=["instance"]))
        response = self.client.get(url)

        self.assertTemplateUsed(response, "instance/get.html")
        self.assertIn('item', response.context)
        self.assertIn('alarms', response.context)
        self.assertIn('events', response.context)
        list_mock.assert_called_with("instance", "bla")

    @mock.patch("tsuru_autoscale.instance.client.alarms_by_instance")
    @mock.patch("tsuru_autoscale.instance.client.get")
    def test_get(self, list_mock, alarms_by_instance_mock):
        json_mock = mock.Mock()
        json_mock.json.return_value = {"Name": "instance"}
        list_mock.return_value = json_mock
        url = "{}?TSURU_TOKEN=bla".format(reverse("instance-get", args=["instance"]))
        response = self.client.get(url)

        self.assertTemplateUsed(response, "instance/get.html")
        self.assertIn('item', response.context)
        self.assertIn('alarms', response.context)
        self.assertIn('events', response.context)
        list_mock.assert_called_with("instance", "bla")


class ClientTestCase(TestCase):
    def setUp(self):
        httpretty.enable()

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def test_list(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.GET,
            "http://autoscalehost.com/service/instance",
        )

        client.list("token")

    def test_get(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.GET,
            "http://autoscalehost.com/service/instance/name",
        )

        client.get("name", "token")

    def test_alarms_by_instance(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.GET,
            "http://autoscalehost.com/alarm/instance/name",
        )

        client.alarms_by_instance("name", "token")



from django.conf.urls import url


urlpatterns = [
    url(r'^new/$', 'tsuru_autoscale.action.views.new', name='action-new'),
    url(r'^(?P<name>[\w\s-]+)/remove/$', 'tsuru_autoscale.action.views.remove', name='action-remove'),
    url(r'^(?P<name>[\w\s-]+)/$', 'tsuru_autoscale.action.views.get', name='action-get'),
    url(r'^$', 'tsuru_autoscale.action.views.list', name='action-list'),
]
